@@ -113,9 +113,11 @@ _gdk_windowing_init (void)
   _cf_utf8_string = RegisterClipboardFormat ("UTF8_STRING");
 
   _utf8_string = gdk_atom_intern ("UTF8_STRING", FALSE);
-  _text_uri_list = gdk_atom_intern ("text/uri-list", FALSE);
   _targets = gdk_atom_intern ("TARGETS", FALSE);
 
+  _text_uri_list = gdk_atom_intern ("text/uri-list", FALSE);
+  _image_bmp = gdk_atom_intern ("image/bmp", FALSE);
+
   _local_dnd = gdk_atom_intern ("LocalDndSelection", FALSE);
   _gdk_win32_dropfiles = gdk_atom_intern ("DROPFILES_DND", FALSE);
   _gdk_ole2_dnd = gdk_atom_intern ("OLE2_DND", FALSE);
@@ -251,7 +253,7 @@ gdk_notify_startup_complete (void)
  */
 static gchar *
 static_printf (const gchar *format,
-			 ...)
+	       ...)
 {
   static gchar buf[10000];
   gchar *msg;
@@ -904,6 +906,91 @@ _gdk_win32_message_to_string (UINT msg)
   return NULL;
 }
 
+gchar *
+_gdk_win32_key_to_string (LONG lParam)
+{
+  char buf[100];
+  gchar *keyname_utf8;
+
+  if (GetKeyNameText (lParam, buf, sizeof (buf)) &&
+      (keyname_utf8 = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL)) != NULL)
+    {
+      gchar *retval = static_printf ("%s", keyname_utf8);
+
+      g_free (keyname_utf8);
+
+      return retval;
+    }
+
+  return static_printf ("unk-%#lx", lParam);
+}
+      
+gchar *
+_gdk_win32_cf_to_string (UINT format)
+{
+  char buf[100];
+
+  switch (format)
+    {
+#define CASE(x) case CF_##x: return #x
+      CASE (BITMAP);
+      CASE (DIB);
+#ifdef CF_DIBV5
+      CASE (DIBV5);
+#endif
+      CASE (DIF);
+      CASE (DSPBITMAP);
+      CASE (DSPENHMETAFILE);
+      CASE (DSPMETAFILEPICT);
+      CASE (DSPTEXT);
+      CASE (ENHMETAFILE);
+      CASE (HDROP);
+      CASE (LOCALE);
+      CASE (METAFILEPICT);
+      CASE (OEMTEXT);
+      CASE (OWNERDISPLAY);
+      CASE (PALETTE);
+      CASE (PENDATA);
+      CASE (RIFF);
+      CASE (SYLK);
+      CASE (TEXT);
+      CASE (WAVE);
+      CASE (TIFF);
+      CASE (UNICODETEXT);
+    default:
+      if (format >= CF_GDIOBJFIRST &&
+	  format <= CF_GDIOBJLAST)
+	return static_printf ("CF_GDIOBJ%d", format - CF_GDIOBJFIRST);
+      if (format >= CF_PRIVATEFIRST &&
+	  format <= CF_PRIVATELAST)
+	return static_printf ("CF_PRIVATE%d", format - CF_PRIVATEFIRST);
+      if (GetClipboardFormatName (format, buf, sizeof (buf)))
+	return static_printf ("%s", buf);
+      else
+	return static_printf ("unk-%#lx", format);
+    }
+}
+      
+gchar *
+_gdk_win32_data_to_string (const guchar *data,
+			   int           nbytes)
+{
+  GString *s = g_string_new ("");
+  int i;
+  gchar *retval;
+
+  for (i = 0; i < nbytes; i++)
+    if (data[i] >=' ' && data[i] <= '~')
+      g_string_append_printf (s, "%c  ", data[i]);
+    else
+      g_string_append_printf (s, "%02X ", data[i]);
+
+  retval = static_printf ("%s", s->str);
+  g_string_free (s, TRUE);
+
+  return retval;
+}
+
 gchar *
 _gdk_win32_rect_to_string (const RECT *rect)
 {
