@@ -24,7 +24,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include "config.h"
+#include <config.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -40,6 +40,7 @@
 #include "gdkdisplay-x11.h"
 
 #include <gdk/gdkinternals.h>
+#include "gdkalias.h"
 
 typedef struct
 {
@@ -256,13 +257,13 @@ gdk_bitmap_create_from_data (GdkDrawable *drawable,
 }
 
 GdkPixmap*
-gdk_pixmap_create_from_data (GdkDrawable *drawable,
-			     const gchar *data,
-			     gint         width,
-			     gint         height,
-			     gint         depth,
-			     GdkColor    *fg,
-			     GdkColor    *bg)
+gdk_pixmap_create_from_data (GdkDrawable    *drawable,
+			     const gchar    *data,
+			     gint            width,
+			     gint            height,
+			     gint            depth,
+			     const GdkColor *fg,
+			     const GdkColor *bg)
 {
   GdkPixmap *pixmap;
   GdkDrawableImplX11 *draw_impl;
@@ -429,3 +430,6 @@ gdk_pixmap_lookup_for_display (GdkDisplay *display, GdkNativeWindow anid)
   g_return_val_if_fail (GDK_IS_DISPLAY (display), NULL);
   return (GdkPixmap*) gdk_xid_table_lookup_for_display (display, anid);
 }
+
+#define __GDK_PIXMAP_X11_C__
+#include  "gdkaliasdef.c"
