@@ -23,6 +23,7 @@
 #include "gtkintl.h"
 #include "gtkmarshalers.h"
 #include "gtktreeprivate.h"
+#include "gtkalias.h"
 
 static void gtk_cell_renderer_toggle_get_property  (GObject                    *object,
 						    guint                       param_id,
@@ -480,3 +481,6 @@ gtk_cell_renderer_toggle_set_active (GtkCellRendererToggle *toggle,
 
   g_object_set (toggle, "active", setting ? TRUE : FALSE, NULL);
 }
+
+#define __GTK_CELL_RENDERER_TOGGLE_C__
+#include "gtkaliasdef.c"
