@@ -1,5 +1,5 @@
 /* GTK - The GIMP Toolkit
- * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
+ * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Jsh MacDonald
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,6 +24,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include "gtkintl.h"
 #include "gtkmenu.h"
 #include "gtkmenuitem.h"
@@ -34,6 +35,8 @@
 #include "gtkoptionmenu.h"
 #define GTK_DISABLE_DEPRECATED
 
+#include "gtkalias.h"
+
 #define CHILD_LEFT_SPACING        4
 #define CHILD_RIGHT_SPACING       1
 #define CHILD_TOP_SPACING         1
@@ -337,6 +340,7 @@ gtk_option_menu_set_history (GtkOptionMenu *option_menu,
  * items are numbered from top to bottom, starting with 0. 
  * 
  * Return value: index of the selected menu item, or -1 if there are no menu items
+ * Deprecated: Use #GtkComboBox instead.
  **/
 gint
 gtk_option_menu_get_history (GtkOptionMenu *option_menu)
@@ -1058,3 +1062,6 @@ gtk_option_menu_scroll_event (GtkWidget          *widget,
   return TRUE;
 }
 
+#define __GTK_OPTION_MENU_C__
+#include "gtkaliasdef.c"
+
